@@ -103,6 +103,9 @@ char *read_sysinfo(void) {
 #ifdef PLATFORM_HPUX
 		return "HP/UX";
 #endif
+#ifdef PLATFORM_GNU
+		return "GNU";
+#endif
 #ifdef PLATFORM_UNKNOWN
 		return "unknown";
 #endif
@@ -111,8 +114,29 @@ char *read_sysinfo(void) {
 
 #ifdef PLATFORM_LINUX
 time_t read_uptime(void) {
+	struct timespec ts;
+	FILE *f;
+	double upseconds = 0;
 	struct sysinfo	si;
 
+
+	if (clock_gettime(CLOCK_BOOTTIME, &ts) == 0)
+		return ts.tv_sec;
+
+
+	/* clock_gettime() failed */
+	f=fopen("/proc/uptime", "r");
+	if (f > 0) {
+		if (fscanf(f, "%lf", &upseconds) > 0) {
+			fclose(f);
+			return((time_t)upseconds);
+		}
+		fclose(f);
+	}
+
+
+	/* reading of /proc/uptime failed */
+
 	/* Until jiffies is declared to something different than unsigned long
 	 * in the kernel sources, this value will probably on all 32b platforms
 	 * wrap past approx. 497 days. This also applies to reading this value
@@ -148,10 +172,10 @@ time_t read_uptime(void) {
 #ifdef PLATFORM_SOLARIS
 time_t read_uptime(void) {
 	int fd;
-	struct utmp ut;
+	struct utmpx ut;
 	int found=0;
 
-	fd = open (UTMP_FILE, O_RDONLY);
+	fd = open(UTMPX_FILE, O_RDONLY);
 	if (fd >= 0) {
 		while (!found) {
 			if (read(fd, &ut, sizeof(ut)) < 0) {
@@ -163,7 +187,7 @@ time_t read_uptime(void) {
 		close(fd);
 	}
 
-	if (found == 1) return time(0) - ut.ut_time;
+	if (found == 1) return time(0) - ut.ut_tv.tv_sec;
 
 	return 0;
 }
@@ -177,7 +201,7 @@ time_t read_uptime(void) {
 }
 #endif
 
-#ifdef PLATFORM_UNKNOWN
+#if defined(PLATFORM_UNKNOWN) || defined(PLATFORM_GNU)
 time_t read_uptime(void) {
 /*
  * This is a quick and inaccurate hack calculating the uptime from the
@@ -215,20 +239,46 @@ void read_records(time_t current) {
 	time_t utime, btime;
 	long l_utime, l_btime;
 	char buf[256], sys[SYSMAX+1];
+	struct stat filestat, filestatold;
+	int useold = 0;
 	
-	f = fopen(FILE_RECORDS, "r");
-	if (!f) {
-		f = fopen(FILE_RECORDS".old", "r");
-		if (!f) return;
+	if (stat(FILE_RECORDS, &filestat))
+		useold = 1;
+	if (stat(FILE_RECORDS".old", &filestatold))
+		useold = -1;
 
-		printf("uptimed: reading from backup database %s.old\n", FILE_RECORDS);
+	/* assume that backupdb larger than normal db means normal is corrupted */
+	if (!useold && (filestat.st_size < filestatold.st_size))
+		useold = 1;
+
+dbtry:
+	switch (useold) {
+		case 0:
+			f = fopen(FILE_RECORDS, "r");
+			break;
+		case 1:
+			f = fopen(FILE_RECORDS".old", "r");
+			printf("uptimed: reading from backup database %s.old\n", FILE_RECORDS);
+			break;
+		default:
+			/* this should probably terminate uptimed somehow */
+			printf("uptimed: no useable database found.\n");
+			return;
+	}
+			
+	if (!f) {
+		printf("uptimed: error opening database for reading.\n");
+		return;
 	}
 	
 	fgets(str, sizeof(str), f);
 	while (!feof(f)) {
 		/* Check for validity of input string. */
 		if (sscanf(str, "%ld:%ld:%[^]\n]", &l_utime, &l_btime, buf) != 3) {
-			/* Skip this entry. Do we want feedback here? */
+			/* database is corrupted */
+			fclose(f);
+			useold++;
+			goto dbtry;
 		} else {
 			utime = (time_t)l_utime;
 			btime = (time_t)l_btime;
@@ -268,34 +318,11 @@ void save_records(int max, time_t log_threshold) {
 	rename(FILE_RECORDS".tmp", FILE_RECORDS);
 }
 
-#ifdef PLATFORM_LINUX
+#if defined(PLATFORM_LINUX) || defined(PLATFORM_BSD) || defined(PLATFORM_GNU)
 int createbootid(void) {
-	FILE *f;
-	char str[256];
-	time_t bootid = 0;
-	
-	f=fopen("/proc/stat", "r");
-	if (!f) {
-		printf ("Error opening /proc file. Can not determine bootid, exiting!\n"); exit(-1);
-	} else {
-		fgets(str, sizeof(str), f);
-		while (!feof(f)) {
-			if (strstr(str, "btime")) {
-				bootid=atoi(str+6);
-				break;
-			}
-			fgets(str, sizeof(str), f);
-		}
-		fclose(f);
-	}
-	
-	f = fopen(FILE_BOOTID, "w");
-	if (!f) {
-		printf("Error writing bootid file, exiting!\n");  exit(-1);
-	} else {
-		fprintf(f, "%ld\n", bootid);
-		fclose(f);
-	}
+	/* these platforms doesn't need to create a bootid file.
+	 * readbootid() fetches it directly from the system every time.
+	 */
 	return 0;
 }
 #endif
@@ -305,11 +332,11 @@ int createbootid(void) {
 int createbootid(void) {
 	FILE *f;
 	int fd;
-	struct utmp ut;
+	struct utmpx ut;
 	int found = 0;
 	time_t bootid = 0;
 
-	fd = open (UTMP_FILE, O_RDONLY);
+	fd = open (UTMPX_FILE, O_RDONLY);
 	if (fd >= 0) {
 		while(!found) {
 			if (read(fd, &ut, sizeof(ut)) < 0) {
@@ -321,7 +348,7 @@ int createbootid(void) {
 		close(fd);
 	}
 
-	if (found == 1) bootid = ut.ut_time;
+	if (found == 1) bootid = ut.ut_tv.tv_sec;
 
 	f = fopen(FILE_BOOTID, "w");
 	if (!f) {
@@ -383,6 +410,31 @@ time_t readbootid(void) {
 		bootid = boottime.tv_sec;
 	}
 
+	return bootid;
+#elif defined(PLATFORM_LINUX) || defined(PLATFORM_GNU)
+	FILE *f;
+	char str[256];
+	time_t bootid = 0;
+
+	f=fopen("/proc/stat", "r");
+	if (!f) {
+		printf ("Error opening /proc/stat file. Can not determine bootid, exiting!\n");
+		exit(-1);
+	} else {
+		fgets(str, sizeof(str), f);
+		while (!feof(f)) {
+			if (strstr(str, "btime")) {
+				bootid=atoi(str+6);
+				break;
+			}
+			fgets(str, sizeof(str), f);
+		}
+		fclose(f);
+	}
+	if (bootid == 0) {
+		printf ("Parsing btime from /proc/stat failed. Can not determine bootid, exiting!\n");
+		exit(-1);
+	}
 	return bootid;
 #else
 	FILE *f;
